using Pulumi;
using Aws = Pulumi.Aws;
using ElasticSearch = Pulumi.Aws.ElasticSearch;
using Pulumi.Aws.ElasticSearch.Inputs;
using System.IO;
using Ionic.Zip;
using System.Net.Http;

class MyStack : Stack
{
    private const string Prefix = "test-05-";
    private string PrefixName(string name)
    {
        return $"{Prefix}{name}";
    }

    public MyStack()
    {
        HttpClient httpClient = new HttpClient()
        {
            BaseAddress = new System.Uri("https://api.ipify.org/")
        };

        string myIPAddress = httpClient.GetStringAsync("?format=text").GetAwaiter().GetResult();

        File.Delete("index.zip");
        using (ZipFile zip = new ZipFile())
        {
            zip.AddDirectory("lambda");
            zip.Save("index.zip");
        }

        // .NET ZipFile does NOT create a zip that AWS considers valid see here https://aws.amazon.com/premiumsupport/knowledge-center/lambda-deployment-package-errors/  
        //ZipFile.CreateFromDirectory("lambda", "index.zip", CompressionLevel.Optimal, false);

        var elasticsearch_indexing_role = new Aws.Iam.Role(PrefixName("elasticsearch_indexing_role"), new Aws.Iam.RoleArgs
        {
            Name = PrefixName("elasticsearch_indexing_role"),
            AssumeRolePolicy = @"{
                                    ""Version"": ""2012-10-17"",
                                    ""Statement"": [
                                        {
                                        ""Action"": ""sts:AssumeRole"",
                                        ""Principal"": {
                                            ""Service"": ""lambda.amazonaws.com""
                                        },
                                        ""Effect"": ""Allow"",
                                        ""Sid"": """"
                                        }
                                    ]
                                }",
        });

        var lambdaKinesisPolicyAttachment = new Aws.Iam.PolicyAttachment(PrefixName("lambdaKinesisPolicyAttachment"), new Aws.Iam.PolicyAttachmentArgs
        {
            Name = PrefixName("lambdaKinesisPolicyAttachment"),
            Roles =
            {
                elasticsearch_indexing_role.Name
            },
            PolicyArn = "arn:aws:iam::aws:policy/service-role/AWSLambdaKinesisExecutionRole",
        });

        var elasticsearch_kinesis = new Aws.Kinesis.Stream(PrefixName("elasticsearch_kinesis"), new Aws.Kinesis.StreamArgs
        {
            Name = PrefixName("elasticsearch_kinesis"),
            RetentionPeriod = 24,
            ShardCount = 1,
            ShardLevelMetrics =
            {
                "IncomingBytes",
                "OutgoingBytes",
            },
        });

        string esDomainName = PrefixName("elasticsearch");
        var config = new Config();
        var currentRegion = Output.Create(Aws.GetRegion.InvokeAsync());
        var currentCallerIdentity = Output.Create(Aws.GetCallerIdentity.InvokeAsync());
        var esDomain = new ElasticSearch.Domain(esDomainName, new ElasticSearch.DomainArgs
        {
            DomainName = esDomainName,
            ClusterConfig = new ElasticSearch.Inputs.DomainClusterConfigArgs
            {
                InstanceType = "t2.small.elasticsearch",
            },
            EbsOptions = new DomainEbsOptionsArgs()
            {
                EbsEnabled = true,
                VolumeSize = 10,
                VolumeType = "gp2"
            },
            ElasticsearchVersion = "7.8",
            AccessPolicies = Output.Tuple(currentRegion, currentCallerIdentity, elasticsearch_indexing_role.Arn).Apply(values =>
            {
                var currentRegion = values.Item1;
                var currentCallerIdentity = values.Item2;
                return $@"
                {{
                    ""Version"": ""2012-10-17"",
                    ""Statement"": [
                        {{
                            ""Effect"": ""Allow"",
                            ""Principal"": {{
                                ""AWS"": ""{values.Item3}""
                            }},
                            ""Action"": ""es:*"",
                            ""Resource"": ""arn:aws:es:{currentRegion.Name}:{currentCallerIdentity.AccountId}:domain/{esDomainName}/*""
                        }},
                        {{
                            ""Action"": ""es:*"",
                            ""Principal"": {{
                                ""AWS"": ""*""
                            }},
                            ""Effect"": ""Allow"",
                            ""Resource"": ""arn:aws:es:{currentRegion.Name}:{currentCallerIdentity.AccountId}:domain/{esDomainName}/*"",
                            ""Condition"": {{
                                ""IpAddress"": {{""aws:SourceIp"": [""{myIPAddress}""]}}
                            }}
                        }}
                    ]
                }}
                ";
            }),
        });
        this.ESDomainEndpoint = esDomain.Endpoint;

        var lambdaEnvironmentVariables = new Aws.Lambda.Inputs.FunctionEnvironmentArgs();
        lambdaEnvironmentVariables.Variables.Add("esUrl", esDomain.Endpoint);
        
        var elasticsearch_indexing_function = new Aws.Lambda.Function(PrefixName("elasticsearch_indexing_function"), new Aws.Lambda.FunctionArgs
        {
            Handler = "index.handler",
            MemorySize = 128,
            Name = PrefixName("elasticsearch_indexing_function"),
            Publish = false,
            ReservedConcurrentExecutions = -1,
            Role = elasticsearch_indexing_role.Arn,
            Runtime = "nodejs12.x",
            Timeout = 4,
            Code = new FileArchive("index.zip"),
            Environment = lambdaEnvironmentVariables
        });

        var elasticsearch_indexing_event_source = new Aws.Lambda.EventSourceMapping(PrefixName("elasticsearch_indexing_event_source"), new Aws.Lambda.EventSourceMappingArgs
        {
            EventSourceArn = elasticsearch_kinesis.Arn,
            FunctionName = elasticsearch_indexing_function.Arn,
            StartingPosition = "LATEST",
        });

    }
    [Output]
    public Output<string> ESDomainEndpoint { get; set; }

}
